==> toxiclibs-clj-0.2.0-20120309.162952-1.jar: toxi/geom/aabb.clj <==
(ns toxi.geom.aabb
  (:require
    [toxi.geom.utils :as utils]
    [toxi.math.core :as math])
  (:use
    [toxi.geom.protocols]
    [toxi.geom.core :only [vec3d vec2d aabb sphere triangle-mesh3d]]
    [toxi.geom.vec3d])
  (:import
    [toxi.geom.types AABB]))

(defn aabb-from-minmax
  [mi mx]
  (aabb (interpolate mi mx 0.5)
        (scale-n
          (vec3d (- (:x mx) (:x mi))
                 (- (:y mx) (:y mi))
                 (- (:z mx) (:z mi)))
          0.5)))

(defn- aabb-min
  {:added "0.1"
   :java-id "toxi.geom.AABB.getMin"}
  [aabb] (utils/swizzle (sub aabb (:extent aabb)) :xyz))

(defn- aabb-max
  {:added "0.1"
   :java-id "toxi.geom.AABB.getMin"}
  [aabb] (utils/swizzle (add aabb (:extent aabb)) :xyz))

(defn- aabb-contains-point?
  {:added "0.1"
   :java-id "toxi.geom.AABB.containsPoint"}
  [aabb p]
  (let [min (aabb-min aabb) max (aabb-max aabb)
        x1 (:x min) y1 (:y min) z1 (:z min)
        x2 (:x max) y2 (:y max) z2 (:z max)
        x (:x p) y (:y p) z (:z p)]
    (and (>= x x1) (>= y y1) (>= z z1) (<= x x2) (<= y y2) (<= z z2))))

(defn- aabb-random-point
  [aabb]
  (let [mi (aabb-min aabb) mx (aabb-max aabb)]
    (vec3d (math/random (:x mi) (:x mx))
           (math/random (:y mi) (:y mx))
           (math/random (:z mi) (:z mx)))))

(defn- aabb-centroid
  [aabb] (utils/swizzle aabb :xyz))

(defn- aabb->mesh
  [aabb]
  (let [a (aabb-min aabb) g (aabb-max aabb)
        b (vec3d (:x a) (:y a) (:z g))
        c (vec3d (:x g) (:y a) (:z g))
        d (vec3d (:x g) (:y a) (:z a))
        e (vec3d (:x a) (:y g) (:z a))
        f (vec3d (:x a) (:y g) (:z g))
        h (vec3d (:x g) (:y g) (:z a))
        ua (vec2d 0 0) ub (vec2d 1 0)
        uc (vec2d 1 1) ud (vec2d 0 1)]
    (add-faces (triangle-mesh3d)
       [[a b f {:uv [ud uc ub]}] [a f e {:uv [ud ub ua]}] ;left
        [b c g {:uv [ud uc ub]}] [b g f {:uv [ud ub ua]}] ;front
        [c d h {:uv [ud uc ub]}] [c h g {:uv [ud ub ua]}] ;right
        [d a e {:uv [ud uc ub]}] [d e h {:uv [ud ub ua]}] ;back
        [e f h {:uv [ua ud ub]}] [f g h {:uv [ud uc ub]}] ;top
        [a d b {:uv [ud uc ua]}] [b d c {:uv [ua uc ub]}]]))) ;bottom

(defn- aabb-bsphere
  [aabb]
  (sphere (utils/swizzle aabb :xyz) (mag (:extent aabb))))

(defn extend-aabb
  [type]
  (extend-vec3d type)
  (extend type
    IShape {
      :bounds identity
      :centroid aabb-centroid
      :contains-point? aabb-contains-point?
      :random-point aabb-random-point
    }
    IShape3D {
      :bounding-sphere aabb-bsphere
      :->mesh aabb->mesh
    }
    IAABB {
      :bounds-min aabb-min
      :bounds-max aabb-max
    }
  ))

(extend-aabb AABB)

==> toxiclibs-clj-0.2.0-20120314.000135-3.jar: toxi/geom/aabb.clj <==
(ns toxi.geom.aabb
  (:require
    [toxi.geom.utils :as utils]
    [toxi.math.core :as math])
  (:use
    [toxi.geom.core]
    [toxi.geom.vec3d])
  (:import
    [toxi.geom.types AABB]))

(defn aabb-from-minmax
  [mi mx]
  (aabb (interpolate mi mx 0.5)
        (scale-n
          (vec3d (- (:x mx) (:x mi))
                 (- (:y mx) (:y mi))
                 (- (:z mx) (:z mi)))
          0.5)))

(defn- aabb-min
  {:added "0.1"
   :java-id "toxi.geom.AABB.getMin"}
  [aabb] (utils/swizzle (sub aabb (:extent aabb)) :xyz))

(defn- aabb-max
  {:added "0.1"
   :java-id "toxi.geom.AABB.getMin"}
  [aabb] (utils/swizzle (add aabb (:extent aabb)) :xyz))

(defn- aabb-contains-point?
  {:added "0.1"
   :java-id "toxi.geom.AABB.containsPoint"}
  [aabb p]
  (let [min (aabb-min aabb) max (aabb-max aabb)
        x1 (:x min) y1 (:y min) z1 (:z min)
        x2 (:x max) y2 (:y max) z2 (:z max)
        x (:x p) y (:y p) z (:z p)]
    (and (>= x x1) (>= y y1) (>= z z1) (<= x x2) (<= y y2) (<= z z2))))

(defn- aabb-random-point
  [aabb]
  (let [mi (aabb-min aabb) mx (aabb-max aabb)]
    (vec3d (math/random (:x mi) (:x mx))
           (math/random (:y mi) (:y mx))
           (math/random (:z mi) (:z mx)))))

(defn- aabb-centroid
  [aabb] (utils/swizzle aabb :xyz))

(defn- aabb->mesh
  [aabb]
  (let [a (aabb-min aabb) g (aabb-max aabb)
        b (vec3d (:x a) (:y a) (:z g))
        c (vec3d (:x g) (:y a) (:z g))
        d (vec3d (:x g) (:y a) (:z a))
        e (vec3d (:x a) (:y g) (:z a))
        f (vec3d (:x a) (:y g) (:z g))
        h (vec3d (:x g) (:y g) (:z a))
        ua (vec2d 0 0) ub (vec2d 1 0)
        uc (vec2d 1 1) ud (vec2d 0 1)]
    (add-faces (triangle-mesh3d)
       [[a b f {:uv [ud uc ub]}] [a f e {:uv [ud ub ua]}] ;left
        [b c g {:uv [ud uc ub]}] [b g f {:uv [ud ub ua]}] ;front
        [c d h {:uv [ud uc ub]}] [c h g {:uv [ud ub ua]}] ;right
        [d a e {:uv [ud uc ub]}] [d e h {:uv [ud ub ua]}] ;back
        [e f h {:uv [ua ud ub]}] [f g h {:uv [ud uc ub]}] ;top
        [a d b {:uv [ud uc ua]}] [b d c {:uv [ua uc ub]}]]))) ;bottom

(defn- aabb-bsphere
  [aabb]
  (sphere (utils/swizzle aabb :xyz) (mag (:extent aabb))))

(defn extend-aabb
  [type]
  (extend-vec3d type)
  (extend type
    IShape {
      :bounds identity
      :centroid aabb-centroid
      :contains-point? aabb-contains-point?
      :random-point aabb-random-point
    }
    IShape3D {
      :bounding-sphere aabb-bsphere
      :->mesh aabb->mesh
    }
    IAABB {
      :bounds-min aabb-min
      :bounds-max aabb-max
    }
  ))

(extend-aabb AABB)
